@@ -64,16 +64,18 @@ class Namespace(object):
     
   def __enter__(self):
     if self.ontology is None: raise ValueError("Cannot assert facts in this namespace: it is not linked to an ontology! (it is probably a global namespace created by get_namespace(); please use your_ontology.get_namespace() instead)")
-    if self.world.graph: self.world.graph.acquire_write_lock()
+    if self.world.graph:
+      self.world.graph.acquire_write_lock()
     l = CURRENT_NAMESPACES.get()
     if l is None: CURRENT_NAMESPACES.set([self])
     else:         l.append(self)
     return self
-    
+  
   def __exit__(self, exc_type = None, exc_val = None, exc_tb = None):
     del CURRENT_NAMESPACES.get()[-1]
-    if self.world.graph: self.world.graph.release_write_lock()
-    
+    if self.world.graph:
+      self.world.graph.release_write_lock()
+      
   def __repr__(self): return """%s.get_namespace("%s")""" % (self.ontology, self.base_iri)
   
   def __getattr__(self, attr): return self.world["%s%s" % (self.base_iri, attr)] #return self[attr]
@@ -250,14 +252,16 @@ WHERE q1.p=? AND q1.o=?
     l = []
     while bnode and (bnode != rdf_nil):
       first, d = self._get_triple_sp_od(bnode, rdf_first)
-      if first != rdf_nil: l.append(self._to_python(first, d))
+      if not ((first == rdf_nil) and (d is None)): l.append(self._to_python(first, d))
       bnode = self._get_obj_triple_sp_o(bnode, rdf_rest)
     return l
   
   def _parse_list_as_rdf(self, bnode):
     while bnode and (bnode != rdf_nil):
       first, d = self._get_triple_sp_od(bnode, rdf_first)
-      if first != rdf_nil: yield first, d
+      if not ((first == rdf_nil) and (d is None)): yield first, d
+      #if (first == rdf_nil) and (d is None): return
+      #yield first, d
       bnode = self._get_obj_triple_sp_o(bnode, rdf_rest)
       
   def _to_python(self, o, d = None, main_type = None, main_onto = None, default_to_none = False):
@@ -441,9 +445,9 @@ class World(_GraphManager):
       
     if filename:
       self.set_backend(backend, filename, dbname, **kargs)
-
-    self.get_ontology("http://anonymous/") # Pre-create, in order to avoird creation during a reading sequence
       
+    self.get_ontology("http://anonymous/") # Pre-create, in order to avoird creation during a reading sequence
+    
   def set_backend(self, backend = "sqlite", filename = ":memory:", dbname = "owlready2_quadstore", **kargs):
     if   backend == "sqlite":
       from owlready2.triplelite import Graph
@@ -528,7 +532,7 @@ class World(_GraphManager):
       else:                                             base_iri = base_iri = "%s#" % base_iri
     if base_iri in self.ontologies: return self.ontologies[base_iri]
     return (OntologyClass or Ontology)(self, base_iri)
-  
+    
   def get_namespace(self, base_iri, name = "", NamespaceClass = None):
     if (not base_iri.endswith("/")) and (not base_iri.endswith("#")) and (not base_iri.endswith(":")):
       if   ("%s#" % base_iri) in self.ontologies: base_iri = base_iri = "%s#" % base_iri
@@ -589,7 +593,7 @@ class World(_GraphManager):
       ontology = ontology0
       if not ontology:
         raise ValueError("Cannot add triples outside a 'with' block. Please start a 'with' block to indicate in which ontology the new triple is added, or include a 'WITH <onto_IRI>' statement in SPARQL.")
-      
+
     sub = None
     if   (s > 0) and (s in self.world._entities): sub = self._entities[s]
     elif (s < 0) and (s in ontology._bnodes):     sub = ontology._bnodes[s]
@@ -600,7 +604,7 @@ class World(_GraphManager):
         except: pass
         
       elif d is None:
-        if o > 0: obj = self._load_by_storid(o)
+        if o > 0: obj = self._get_by_storid(o)
         else:     obj = self._parse_bnode(o)
         
         if not obj is None:
@@ -792,6 +796,8 @@ class World(_GraphManager):
      
 class Ontology(Namespace, _GraphManager):
   def __init__(self, world, base_iri, name = None):
+    if world.graph: world.graph.acquire_write_lock()
+    
     self.world       = world # Those 2 attributes are required before calling Namespace.__init__
     self._namespaces = weakref.WeakValueDictionary()
     Namespace.__init__(self, self, base_iri, name)
@@ -800,6 +806,7 @@ class Ontology(Namespace, _GraphManager):
     self.storid                = world._abbreviate(base_iri[:-1])
     self._imported_ontologies  = CallbackList([], self, Ontology._import_changed)
     self.metadata              = Metadata(self, self.storid)
+
     
     if world.graph is None:
       self.graph = None
@@ -815,12 +822,12 @@ class Ontology(Namespace, _GraphManager):
     
     if (not LOADING) and (not self.graph is None):
       if not self._has_obj_triple_spo(self.storid, rdf_type, owl_ontology):
-        if self.world.graph: self.world.graph.acquire_write_lock()
         self._add_obj_triple_spo(self.storid, rdf_type, owl_ontology)
-        if self.world.graph: self.world.graph.release_write_lock()
         
     if not self.world._rdflib_store is None: self.world._rdflib_store._add_onto(self)
     
+    if world.graph: world.graph.release_write_lock()
+    
   def destroy(self):
     self.world.graph.acquire_write_lock()
     del self.world.ontologies[self.base_iri]
@@ -860,11 +867,6 @@ class Ontology(Namespace, _GraphManager):
     if not r is None: return r
     return Namespace(self, base_iri, name or base_iri[:-1].rsplit("/", 1)[-1])
   
-  #def __exit__(self, exc_type = None, exc_val = None, exc_tb = None):
-  #  Namespace.__exit__(self, exc_type, exc_val, exc_tb)
-  #  if not self.loaded:
-  #    self.loaded = True
-  #    if self.graph: self.graph.set_last_update_time(time.time())
   def __enter__(self): # Do this in __enter__ and not __exit__, because set_last_update_time() modify the database. Modifying the database in __exit__ prevents calling World.save() inside the 'with onto:' block. 
     Namespace.__enter__(self)
     if not self.loaded:
@@ -880,7 +882,8 @@ class Ontology(Namespace, _GraphManager):
         
   def load(self, only_local = False, fileobj = None, reload = False, reload_if_newer = False, url = None, **args):
     if self.loaded and (not reload): return self
-
+    
+    
     if   self.base_iri in PREDEFINED_ONTOLOGIES:
       f = os.path.join(os.path.dirname(__file__), "ontos", PREDEFINED_ONTOLOGIES[self.base_iri])
     elif not fileobj:
@@ -891,7 +894,7 @@ class Ontology(Namespace, _GraphManager):
     self.world.graph.acquire_write_lock()
     
     if reload: self._destroy_cached_entities()
-    
+
     new_base_iri = None
     if f.startswith("http:") or f.startswith("https:"):
       if  reload or (self.graph.get_last_update_time() == 0.0): # Never loaded
